from hitchmysql.mysql_service import MySQLDatabase
from hitchmysql.mysql_service import MySQLUser
from hitchmysql.mysql_service import MySQLService
from hitchmysql.mysql_package import MySQLPackage


UNIXPACKAGES = ["libaio1", "libmysqlclient-dev", "libncurses5-dev", ]
